/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.fileUpload;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.unis.config.DevFileMinIoUtil;
import com.unis.enums.DevFileEngineTypeEnum;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.UUID;


/**
 * 文件Service接口
 *
 * @author xuyuxiang
 * @date 2022/2/23 18:27
 **/
public interface DevFileService {

    /**
     * MultipartFile文件上传，返回文件Url
     *
     * @author xuyuxiang
     * @date 2022/4/22 15:53
     **/
    String uploadReturnUrl(MultipartFile file);

    @Service
    class DevFileServiceImpl implements DevFileService {

        @Override
        public String uploadReturnUrl(MultipartFile file) {
            return this.storageFile(file);
        }

        /**
         * 存储文件
         *
         * @author xuyuxiang
         * @date 2022/6/16 16:24
         **/
        private String storageFile(MultipartFile file) {
            // 生成id
            String fileId = UUID.randomUUID().toString();
            return DevFileMinIoUtil.storageFileWithReturnUrl(DevFileMinIoUtil.getDefaultBucketName(), genFileKey(fileId, file), file);
        }

        @SuppressWarnings("all")
        public String genFileKey(String fileId, MultipartFile file) {

            // 获取文件原始名称
            String originalFileName = file.getOriginalFilename();

            // 获取文件后缀
            String fileSuffix = FileUtil.getSuffix(originalFileName);

            // 生成文件的对象名称，格式如:1377109572375810050.docx
            String fileObjectName = fileId + StrUtil.DOT + fileSuffix;

            // 获取日期文件夹，格式如，2021/10/11/
            String dateFolderPath = DateUtil.thisYear() + StrUtil.SLASH +
                    (DateUtil.thisMonth() + 1) + StrUtil.SLASH +
                    DateUtil.thisDayOfMonth() + StrUtil.SLASH;

            // 返回
            return dateFolderPath + fileObjectName;
        }

    }

}
